define(function(require) {

    // Initial Setup
    // -------------

    // get references to Backbone and jQuery libraries
    var $ = require('jquery');
    var Backbone = require("backbone");


    // Backbone's View enhancementes
    // -------------
    Backbone.View.prototype.addSubview = function(view) 
    {
        if (this._subviews)
            this._subviews.push(view);
        else
            this._subviews = [view];
    };
    Backbone.View.prototype.closeSubviews = function(view) 
    {
        if (this._subviews)
            _.each(this._subviews, function(subview) {
                subview.close();
            });
    };
    // function that will be called by the router every time a view must be removed from the DOM 
    Backbone.View.prototype.close = function(options) 
    {
        var opts = options || {};

        // default vars
        if (!opts.removeFromDOM)
            opts.removeFromDOM = false;

        // notify the new view that it is being removed
        this.trigger("removing");
        
        // close all subviews
        this.closeSubviews();
        
        // remove the view from the DOM
        if (opts.removeFromDOM)
            this.remove();

        // remove references to the DOM element of the view (both jQuery and JS objects)
        this.$el = null;
        this.el = null;
    };


    // Backbone's Router enhancementes
    // -------------
    // this two method are specific to jQuery Mobile
    Backbone.Router.prototype.backButton = function () 
    {
        this.back = true;
        window.history.go(-2);
    };
    Backbone.Router.prototype.changePage = function (page, options) 
    {
        var opts = options || {};

        if (!opts.transition)
            opts.transition = $.mobile.defaultPageTransition;

        // every page must have this attribute
        page.$el.attr('data-role', 'page').attr('data-theme', 'c').addClass('scrollable');
        page.render();
        
        // prepend content to page and add it to subviews of page
        if (opts.prependPageContent) {
            page.$el.prepend( opts.prependPageContent.render().$el );
            page.addSubview(opts.prependPageContent);
        }

        // Remove page from DOM when it's being replaced
        page.$el.on('pagehide', function (event, ui) {
            $(this).remove();
        });
        page.$el.on('pageshow', function (event, ui) {
            page.trigger('inDOM');
        });

        // prepend the content of rendered page to the body
        $('body').prepend(page.$el);
        // change the page
        $.mobile.changePage(page.$el, {changeHash: false, transition: opts.transition, reverse: this.back});

        // close last page and update with new page
        if (this._currentPage)
            this._currentPage.close();
        this._currentPage = page;

        this.back = false;
    };
    // Give the ability to cache values on router level.
    Backbone.Router.prototype.setCache = function(name, value) 
    {
        this._cache = this._cache || {};
        this._cache[name] = value;
        return value;
    };
    Backbone.Router.prototype.getCache = function(name, defvalue) 
    {
        if (this.existCache(name))
            return this._cache[name];
        else if (defvalue) {
            this.setCache(name, defvalue);
            return defvalue;
        }
        else 
            return null;
    };
    Backbone.Router.prototype.delCache = function(name) 
    {
        if (this.existCache(name)) {
            this._cache[name] = null;
            delete this._cache[name];
        }
    };
    Backbone.Router.prototype.existCache = function(name) 
    {
        return this._cache && name in this._cache;
    };

});